# Editing Settings of a Guest
use strict;
use warnings;

our (%gui, %signal, %hostspec, %ETruth, %EAudioDriverType, %EAudioControllerType, %ENetworkAttachmentType,
     %ENetworkAdapterType, %osfamily, %osversion, %sensgrp, $EIMachineM);

sub show_shared_dialog() {
    my ($widget) = @_;
    &busy_window($gui{dialogEdit});

    if ($widget eq $gui{buttonEditSharedEdit}) {
        my ($ename, $efolder, $eaccess) = &get_selected($gui{treeviewEditShared});
        $gui{entrySharedFolderPath}->set_text($efolder);
        $gui{entrySharedFolderName}->set_text($ename);
        $gui{checkbuttonSharedRO}->set_active(1) if ($eaccess eq 'Read-Only');
    }

    my $response = $gui{dialogShared}->run;
    $gui{dialogShared}->hide;

    if ($response eq 'ok') {
        my $path = $gui{entrySharedFolderPath}->get_text();
        my $name = $gui{entrySharedFolderName}->get_text();
        if (!$path or !$name) { &show_err_msg('emptynamepath', $gui{messagedialogError}); }
        else {
            remove_shared($widget, $gui{treeviewEditShared}) if ($widget eq $gui{buttonEditSharedEdit});
            IMachine_createSharedFolder($EIMachineM, $name, $path, !$gui{checkbuttonSharedRO}->get_active());
            &tv_shared_fill($EIMachineM);
        }
    }

    &unbusy_window($gui{dialogEdit});
}

sub show_edit_dialog() {
    my ($widget, $treeview) = @_;
    my ($gname, $gos, $IMachine, $status, $gosid) = &get_selected($treeview);
    &busy_window($gui{windowMain});
    my ($ISession, $IMachineM) = &get_mutable_session($IMachine);

    if ($ISession)
    {
        &set_status_msg("Retrieving guest settings for $gname...");
        $gui{dialogEdit}->set_title("Edit Settings - $gname");

        $EIMachineM = $IMachineM;
        $gui{comboboxEditGenOSFam}->signal_handler_block($signal{famedit});
        $gui{comboboxEditGenOSVer}->signal_handler_block($signal{veredit});
        $gui{comboboxEditAudioDriver}->signal_handler_block($signal{audiodrv});
        $gui{comboboxEditAudioCtr}->signal_handler_block($signal{audioctr});
        $gui{comboboxEditNetType1}->signal_handler_block($signal{netadaptype1});
        $gui{comboboxEditNetType2}->signal_handler_block($signal{netadaptype2});
        $gui{comboboxEditNetType3}->signal_handler_block($signal{netadaptype3});
        $gui{comboboxEditNetType4}->signal_handler_block($signal{netadaptype4});
        $gui{comboboxEditNetAttach1}->signal_handler_block($signal{netattach1});
        $gui{comboboxEditNetAttach2}->signal_handler_block($signal{netattach2});
        $gui{comboboxEditNetAttach3}->signal_handler_block($signal{netattach3});
        $gui{comboboxEditNetAttach4}->signal_handler_block($signal{netattach4});
        $gui{comboboxEditNetName1}->signal_handler_block($signal{netname1});
        $gui{comboboxEditNetName2}->signal_handler_block($signal{netname2});
        $gui{comboboxEditNetName3}->signal_handler_block($signal{netname3});
        $gui{comboboxEditNetName4}->signal_handler_block($signal{netname4});

        &clr_lstore($gui{liststoreEditGenOSFam}, $gui{liststoreEditGenOSVer}, $gui{liststoreEditAudioDriver},
                    $gui{liststoreEditAudioCtr}, $gui{liststoreEditNetAdapterType1}, $gui{liststoreEditNetAdapterType2},
                    $gui{liststoreEditNetAdapterType3}, $gui{liststoreEditNetAdapterType4}, $gui{liststoreEditNetAttachTo1},
                    $gui{liststoreEditNetAttachTo2}, $gui{liststoreEditNetAttachTo3}, $gui{liststoreEditNetAttachTo4},
                    $gui{liststoreEditNetName1}, $gui{liststoreEditNetName2}, $gui{liststoreEditNetName3},
                    $gui{liststoreEditNetName4}, $gui{liststoreEditSysBoot});
        # These widgets must start insensitive
        &sensitive_off($gui{buttonEditSysBootUp}, $gui{buttonEditSysBootDown});
        &setup_edit_dialog_general($IMachineM, $treeview);
        &setup_edit_dialog_system($IMachineM);
        &setup_edit_dialog_display($IMachineM);
        &setup_edit_dialog_storage($IMachineM);
        &setup_edit_dialog_audio($IMachineM);
        &setup_edit_dialog_network($IMachineM);
        &setup_edit_dialog_serial($IMachineM);
        &setup_edit_dialog_usb($IMachineM);
        &setup_edit_dialog_shared($IMachineM);
        $gui{comboboxEditGenOSFam}->signal_handler_unblock($signal{famedit});
        $gui{comboboxEditGenOSVer}->signal_handler_unblock($signal{veredit});
        $gui{comboboxEditAudioDriver}->signal_handler_unblock($signal{audiodrv});
        $gui{comboboxEditAudioCtr}->signal_handler_unblock($signal{audioctr});
        $gui{comboboxEditNetType1}->signal_handler_unblock($signal{netadaptype1});
        $gui{comboboxEditNetType2}->signal_handler_unblock($signal{netadaptype2});
        $gui{comboboxEditNetType3}->signal_handler_unblock($signal{netadaptype3});
        $gui{comboboxEditNetType4}->signal_handler_unblock($signal{netadaptype4});
        $gui{comboboxEditNetAttach1}->signal_handler_unblock($signal{netattach1});
        $gui{comboboxEditNetAttach2}->signal_handler_unblock($signal{netattach2});
        $gui{comboboxEditNetAttach3}->signal_handler_unblock($signal{netattach3});
        $gui{comboboxEditNetAttach4}->signal_handler_unblock($signal{netattach4});
        $gui{comboboxEditNetName1}->signal_handler_unblock($signal{netname1});
        $gui{comboboxEditNetName2}->signal_handler_unblock($signal{netname2});
        $gui{comboboxEditNetName3}->signal_handler_unblock($signal{netname3});
        $gui{comboboxEditNetName4}->signal_handler_unblock($signal{netname4});
        &set_status_msg("Settings retrieved for $gname");
        my $response = $gui{dialogEdit}->run;
        $gui{dialogEdit}->hide;

        if ($response eq 'ok') {
            IMachine_saveSettings($IMachineM);
            &tv_guest_fill();
        }
        else { IMachine_discardSettings($IMachineM); }

        ISession_close($ISession);
        $EIMachineM = undef;
        $gui{menuitemAttachFloppy} = undef; # These must be freed
        $gui{menuitemAttachDVD} = undef;
        $gui{menuitemAttachHD} = undef;
        $gui{menuAttachFloppy} = undef;
        $gui{menuAttachDVD} = undef;
        $gui{menuAttachHD} = undef;
        $gui{menuAttachAdd} = undef;
    }
    else { &show_err_msg('settings', $gui{messagedialogWarning}); }

    &unbusy_window($gui{windowMain});
}

sub setup_edit_dialog_general() {
    my ($IMachine, $treeview) = @_;
    my $IGuestOSType = IVirtualBox_getGuestOSType($gui{websn}, &get_selected($treeview, 4));
    $gui{entryEditGenName}->set_text(&get_selected($treeview, 0));

    foreach my $fam (sort(keys %osfamily)) {
        my $iter = &append_lstore($gui{liststoreEditGenOSFam}, (0 => "$osfamily{$fam}{description}", 1 => "$fam" ));
        $gui{comboboxEditGenOSFam}->set_active_iter($iter) if ($fam eq $$IGuestOSType{familyId});
    }

    foreach my $ver (sort(@{ $osfamily{$$IGuestOSType{familyId}}{verids} })) {
        my $iter = &append_lstore($gui{liststoreEditGenOSVer}, (0 => "$osversion{$ver}{description}", 1 => "$ver"));
        $gui{comboboxEditGenOSVer}->set_active_iter($iter) if ($ver eq $$IGuestOSType{id});
    }
}

sub setup_edit_dialog_system() {
    my ($IMachine) = @_;
    my $IBIOSSettings = IMachine_getBIOSSettings($IMachine);
    if (IMachine_getFirmwareType($IMachine) =~ m/^EFI/) { $gui{checkbuttonEditSysEFI}->set_active(1); }
    else { $gui{checkbuttonEditSysEFI}->set_active(0); }
    $gui{spinbuttonEditSysMem}->set_range(4, $hostspec{memsize});
    $gui{spinbuttonEditSysMem}->set_value(IMachine_getMemorySize($IMachine));
    $gui{checkbuttonEditSysVTX}->set_active($ETruth{IMachine_getHWVirtExProperty($IMachine, 'Enabled')});
    $gui{checkbuttonEditSysNested}->set_active($ETruth{IMachine_getHWVirtExProperty($IMachine, 'NestedPaging')});
    $gui{hscaleEditSysProcessor}->set_range($hostspec{minguestcpu}, $hostspec{maxhostcpuon} + 1); # +1 works around GTK bug
    $gui{hscaleEditSysProcessor}->set_value(IMachine_getCPUCount($IMachine));
    $gui{checkbuttonEditSysAPIC}->set_active($ETruth{IBIOSSettings_getIOAPICEnabled($IBIOSSettings)});
    $gui{checkbuttonEditSysPAE}->set_active($ETruth{IMachine_getCPUProperty($IMachine, 'PAE')});
    $gui{checkbuttonEditSysUTC}->set_active($ETruth{IMachine_getRTCUseUTC($IMachine)});
    my $pointer = IMachine_getPointingHidType($IMachine);

    if ($pointer eq 'PS2Mouse') { $gui{comboboxEditSysPointing}->set_active(1); }
    elsif ($pointer eq 'USBMouse') { $gui{comboboxEditSysPointing}->set_active(2); }
    elsif ($pointer eq 'USBTablet') { $gui{comboboxEditSysPointing}->set_active(3); }
    elsif ($pointer eq 'ComboMouse') { $gui{comboboxEditSysPointing}->set_active(4); }
    else { $gui{comboboxEditSysPointing}->set_active(0); }

    # Default to maxbootpos+1 to mean 'not set in boot order' but this number needs to be higher than
    # true boot order numbers so the disabled devices appear at the end of the list.
    my %bootorder = (Floppy   => $hostspec{maxbootpos} + 1,
                     DVD      => $hostspec{maxbootpos} + 1,
                     HardDisk => $hostspec{maxbootpos} + 1,
                     Network  => $hostspec{maxbootpos} + 1);
    # Find boot order and set value in hash accordingly. Empty boot slots return 'Null' so skip them
    foreach (1..$hostspec{maxbootpos}) {
        my $bootdev = IMachine_getBootOrder($IMachine,$_);
        next if ($bootdev eq 'Null');
        $bootorder{$bootdev} = $_;
    }
    # Returns hash keys sorted by value (ie boot order). Disabled devices appear at end
    foreach my $dev (sort {$bootorder{$a} cmp $bootorder{$b}} keys %bootorder) {
        if ($bootorder{$dev} == $hostspec{maxbootpos} + 1) { &append_lstore($gui{liststoreEditSysBoot}, (0 => 0, 1 => $dev)); }
        else { &append_lstore($gui{liststoreEditSysBoot}, (0 => 1, 1 => $dev)); }
    }
}

sub setup_edit_dialog_display() {
    my ($IMachine) = @_;
    my $IVRDPServer = IMachine_getVRDPServer($IMachine);
    $gui{spinbuttonEditDispVidMem}->set_range($hostspec{minguestvram}, $hostspec{maxguestvram});
    $gui{spinbuttonEditDispVidMem}->set_value(IMachine_getVRAMSize($IMachine));
    $gui{spinbuttonEditDispMonitor}->set_range(1, $hostspec{maxmonitors});
    $gui{spinbuttonEditDispMonitor}->set_value(IMachine_getMonitorCount($IMachine));
    $gui{checkbuttonEditDisp3D}->set_active($ETruth{IMachine_getAccelerate3DEnabled($IMachine)});
    $gui{checkbuttonEditDisp2D}->set_active($ETruth{IMachine_getAccelerate2DVideoEnabled($IMachine)});
    $gui{checkbuttonEditDispServer}->set_active($ETruth{IVRDPServer_getEnabled($IVRDPServer)});
    $gui{entryEditDispPort}->set_text(IVRDPServer_getPorts($IVRDPServer));
    $gui{entryEditDispAuthTime}->set_text(IVRDPServer_getAuthTimeout($IVRDPServer));
    $gui{checkbuttonEditDispMultiple}->set_active($ETruth{IVRDPServer_getAllowMultiConnection($IVRDPServer)});
    my $authtype = IVRDPServer_getAuthType($IVRDPServer);

    if ($authtype eq 'External') { $gui{comboboxDispAuthMeth}->set_active(1); }
    elsif ($authtype eq 'Guest') { $gui{comboboxDispAuthMeth}->set_active(2); }
    else { $gui{comboboxDispAuthMeth}->set_active(0); }
}

sub setup_edit_dialog_audio() {
    my ($IMachine) = @_;
    my $IAudioAdapter = IMachine_getAudioAdapter($IMachine);
    $gui{checkbuttonEditAudioEnable}->set_active($ETruth{IAudioAdapter_getEnabled($IAudioAdapter)});

    foreach my $driver (sort(keys %EAudioDriverType)) {
        if ($hostspec{os} =~ m/${EAudioDriverType{$driver}{os}}/ or $EAudioDriverType{$driver}{os} eq 'All') {
            my $iter = &append_lstore($gui{liststoreEditAudioDriver}, (0 => $EAudioDriverType{$driver}{desc},
                                                                       1 => $driver ));
            $gui{comboboxEditAudioDriver}->set_active_iter($iter) if ($driver eq IAudioAdapter_getAudioDriver($IAudioAdapter));
        }
    }

    foreach my $ctr (sort(keys %EAudioControllerType)) {
        my $iter = &append_lstore($gui{liststoreEditAudioCtr}, (0 => $EAudioControllerType{$ctr}, 1 => $ctr ));
        $gui{comboboxEditAudioCtr}->set_active_iter($iter) if ($ctr eq IAudioAdapter_getAudioController($IAudioAdapter));
    }
}

sub setup_edit_dialog_network() {
    my ($IMachine) = @_;
    my $IHost = IVirtualBox_getHost($gui{websn});

    foreach my $slot (1..4) {
        my $INetworkAdapter = IMachine_getNetworkAdapter($IMachine,($slot-1));
        my $NetAdapterType = INetworkAdapter_getAdapterType($INetworkAdapter);
        my $AttachType = INetworkAdapter_getAttachmentType($INetworkAdapter);
        my $hostinterface = INetworkAdapter_getHostInterface($INetworkAdapter);
        my @IHostNetworkInterface = IHost_getNetworkInterfaces($IHost);
        $gui{'checkbuttonEditNetEnable' . $slot}->set_active($ETruth{INetworkAdapter_getEnabled($INetworkAdapter)});
        $gui{'checkbuttonEditNetCable' . $slot}->set_active($ETruth{INetworkAdapter_getCableConnected($INetworkAdapter)});
        $gui{'entryEditNetMac' . $slot}->set_text(INetworkAdapter_getMACAddress($INetworkAdapter));
        # Set current adapter type for NICs 1-4
        foreach my $adaptertype (sort(keys %ENetworkAdapterType)) {
            my $iter = &append_lstore($gui{'liststoreEditNetAdapterType' . $slot}, (0 => $ENetworkAdapterType{$adaptertype},
                                                                                    1 => $adaptertype ));
            $gui{'comboboxEditNetType' . $slot}->set_active_iter($iter) if ($adaptertype eq $NetAdapterType);
        }
        # Set current attachment type for NICs 1-4
        foreach my $attach (sort(keys %ENetworkAttachmentType)) {
            my $iter = &append_lstore($gui{'liststoreEditNetAttachTo' . $slot}, (0 => $ENetworkAttachmentType{$attach},
                                                                                 1 => $attach));
            $gui{'comboboxEditNetAttach' . $slot}->set_active_iter($iter) if ($attach eq $AttachType);
        }

        if ($AttachType eq 'Bridged') {
            # Set current bridged network name for NICs 1-4, depending on attachment type
            foreach my $interface (@IHostNetworkInterface) {
                if (IHostNetworkInterface_getInterfaceType($interface) eq 'Bridged') {
                    my $name = IHostNetworkInterface_getName($interface);
                    my $iter = &append_lstore($gui{'liststoreEditNetName' . $slot}, (0 => $name));
                    $gui{'comboboxEditNetName' . $slot}->set_active_iter($iter) if ($name eq $hostinterface);
                }
            }
            &hide_off($gui{'labelEditNetName' . $slot}, $gui{'comboboxEditNetName' . $slot});
            &hide_on($gui{'labelEditNetNameInt' . $slot}, $gui{'entryEditNetNameInt' . $slot});
        }
        elsif ($AttachType eq 'HostOnly') {
            # Set current hostonly network name for NICs 1-4, depending on attachment type
            foreach my $interface (@IHostNetworkInterface) {
                if (IHostNetworkInterface_getInterfaceType($interface) eq 'HostOnly') {
                    my $name = IHostNetworkInterface_getName($interface);
                    my $iter = &append_lstore($gui{'liststoreEditNetName' . $slot}, (0 => $name));
                    $gui{'comboboxEditNetName' . $slot}->set_active_iter($iter) if ($name eq $hostinterface);
                }
            }
            &hide_off($gui{'labelEditNetName' . $slot}, $gui{'comboboxEditNetName' . $slot});
            &hide_on($gui{'labelEditNetNameInt' . $slot}, $gui{'entryEditNetNameInt' . $slot});
        }
        elsif ($AttachType eq 'Internal') {
            my $name = INetworkAdapter_getInternalNetwork($INetworkAdapter);
            $gui{'entryEditNetNameInt' . $slot}->set_text($name);
            &hide_on($gui{'labelEditNetName' . $slot}, $gui{'comboboxEditNetName' . $slot});
            &hide_off($gui{'labelEditNetNameInt' . $slot}, $gui{'entryEditNetNameInt' . $slot});
        }
        #elsif ($AttachType eq 'VDE') {
        #
        #}
        else {
            # All other networking types do not have name associated with them
            &hide_on($gui{'labelEditNetName' . $slot}, $gui{'comboboxEditNetName' . $slot},
                     $gui{'labelEditNetNameInt' . $slot}, $gui{'entryEditNetNameInt' . $slot});
        }
    }
}

sub setup_edit_dialog_serial() {
    my ($IMachine) = @_;

    foreach my $slot (1..2) {
        my $ISerialPort = IMachine_getSerialPort($IMachine, ($slot - 1));
        my $irq = ISerialPort_getIRQ($ISerialPort);
        my $ioport = sprintf('%X', ISerialPort_getIOBase($ISerialPort));
        my $mode = ISerialPort_getHostMode($ISerialPort);
        $gui{'checkbuttonEditSerialEnable' . $slot}->set_active($ETruth{ISerialPort_getEnabled($ISerialPort)});
        $gui{'checkbuttonEditSerialPipe' . $slot}->set_active($ETruth{ISerialPort_getServer($ISerialPort)});
        $gui{'entryEditSerialPath' . $slot}->set_text(ISerialPort_getPath($ISerialPort));
        $gui{'entryEditSerialIRQ' . $slot}->set_text($irq);
        $gui{'entryEditSerialIO' . $slot}->set_text($ioport);
        &sensitive_off($gui{'entryEditSerialIRQ' . $slot}, $gui{'entryEditSerialIO' . $slot});

        if ($irq == 4 and uc($ioport) eq '3F8') { $gui{'comboboxEditSerialPortNum' . $slot}->set_active(0); }
        elsif ($irq == 3 and uc($ioport) eq '2F8') { $gui{'comboboxEditSerialPortNum' . $slot}->set_active(1); }
        elsif ($irq == 4 and uc($ioport) eq '3E8') { $gui{'comboboxEditSerialPortNum' . $slot}->set_active(2); }
        elsif ($irq == 3 and uc($ioport) eq '2E8') { $gui{'comboboxEditSerialPortNum' . $slot}->set_active(3); }
        else {
            $gui{'comboboxEditSerialPortNum' . $slot}->set_active(4);
            &sensitive_on($gui{'entryEditSerialIRQ' . $slot}, $gui{'entryEditSerialIO' . $slot});
        }

        if ($mode eq 'HostPipe') { $gui{'comboboxEditSerialMode' . $slot}->set_active(1); }
        elsif ($mode eq 'HostDevice') { $gui{'comboboxEditSerialMode' . $slot}->set_active(2); }
        elsif ($mode eq 'RawFile') { $gui{'comboboxEditSerialMode' . $slot}->set_active(3); }
        else { $gui{'comboboxEditSerialMode' . $slot}->set_active(0); }
        $gui{'checkbuttonEditSerialPipe' . $slot}->set_active($ETruth{ISerialPort_getServer($ISerialPort)});
    }
}

sub setup_edit_dialog_usb() {
    my ($IMachine) = @_;
    my $IUSBController = IMachine_getUSBController($EIMachineM);
    $gui{checkbuttonEditUSBEnable}->set_active($ETruth{IUSBController_getEnabled($IUSBController)});
    $gui{checkbuttonEditUSBEHCI}->set_active($ETruth{IUSBController_getEnabledEhci($IUSBController)});
}

sub setup_edit_dialog_shared() {
    my ($IMachine) = @_;
    &tv_shared_fill($IMachine);
}

sub setup_edit_dialog_storage() {
    my ($IMachine) = @_;
    &editstor_tree_fill($IMachine);
    $gui{menuAttachAdd} = Gtk2::Menu->new();
    $gui{menuAttachHD} = Gtk2::Menu->new();
    $gui{menuAttachDVD} = Gtk2::Menu->new();
    $gui{menuAttachFloppy} = Gtk2::Menu->new();
    $gui{menuitemAttachHD} = Gtk2::MenuItem->new_with_label('Hard Disks');
    $gui{menuitemAttachDVD} = Gtk2::MenuItem->new_with_label('CDs/DVDs');
    $gui{menuitemAttachFloppy} = Gtk2::MenuItem->new_with_label('Floppy Disks');
    $gui{menuitemAttachHD}->set_submenu($gui{menuAttachHD});
    $gui{menuitemAttachDVD}->set_submenu($gui{menuAttachDVD});
    $gui{menuitemAttachFloppy}->set_submenu($gui{menuAttachFloppy});
    $gui{menuitemAttachHD}->show();
    $gui{menuitemAttachDVD}->show();
    $gui{menuitemAttachFloppy}->show();
    $gui{menuAttachAdd}->append($gui{menuitemAttachHD});
    $gui{menuAttachAdd}->append($gui{menuitemAttachDVD});
    $gui{menuAttachAdd}->append($gui{menuitemAttachFloppy});

    my %IMediumHD = &get_all_media('HardDisk');
    my %IMediumDVD = &get_all_media('DVD');
    my %IMediumFloppy = &get_all_media('Floppy');

    foreach my $medium (sort { lc($a) cmp lc($b) } (keys %IMediumHD)) {
        my $item = Gtk2::MenuItem->new_with_label($medium);
        $gui{menuAttachHD}->append($item);
        $item->show();
        $item->signal_connect(activate => \&editstor_attach_add, $IMediumHD{$medium});
    }

    foreach my $medium (sort { lc($a) cmp lc($b) } (keys %IMediumDVD)) {
        my $item = Gtk2::MenuItem->new_with_label($medium);
        $gui{menuAttachDVD}->append($item);
        $item->show();
        $item->signal_connect(activate => \&editstor_attach_add, $IMediumDVD{$medium});
    }

    foreach my $medium (sort { lc($a) cmp lc($b) } (keys %IMediumFloppy)) {
        my $item = Gtk2::MenuItem->new_with_label($medium);
        $gui{menuAttachFloppy}->append($item);
        $item->show();
        $item->signal_connect(activate => \&editstor_attach_add, $IMediumFloppy{$medium});
    }
}

sub show_attach_menu() {
    my ($widget, $event) = @_;
    $gui{menuAttachAdd}->popup(undef, undef, undef, undef, 0, $event->time) if ($event->button == 1);
    return 0;
}

sub show_ctr_menu() {
    my ($widget, $event) = @_; #$event->time
    $gui{menuCtrAdd}->popup(undef, undef, undef, undef, 0, $event->time) if ($event->button == 1);
    return 0;
}

sub editgen_name() {
    my ($widget) = @_;
    my $name = $widget->get_text();
    IMachine_setName($EIMachineM, $name) if ($name);
}

sub editgen_osfam() {
    my ($combofam, $combover) = @_;
    my $fam = &get_selected($combofam, 1);
    $combofam->signal_handler_block($signal{famedit}); # Block to avoid signal emission when changing
    $combover->signal_handler_block($signal{veredit});
    &clr_lstore($gui{liststoreEditGenOSVer});

    foreach my $ver (sort(@{ $osfamily{$fam}{verids} })) {
        my $iter = &append_lstore($gui{liststoreEditGenOSVer}, (0 => "$osversion{$ver}{description}",
                                                                1 => "$ver" ));
        $combover->set_active_iter($iter) if ($ver eq 'WindowsXP' | $ver eq 'Ubuntu' | $ver eq 'Solaris');
    }

    $combover->set_active(0) if ($combover->get_active() == -1);
    $combofam->signal_handler_unblock($signal{famedit});
    $combover->signal_handler_unblock($signal{veredit});
    $combover->signal_emit('changed'); # Force update of other fields based on OS
}

sub editgen_osver() {
    my ($combover) = @_;
    my $ver = &get_selected($combover, 1);
    IMachine_setOSTypeId($EIMachineM, $ver);
}

sub editsys_boot_toggle() {
    my ($widget, $path_str, $model) = @_;
    my $iter = $model->get_iter(Gtk2::TreePath->new_from_string($path_str));
    my $val = $model->get($iter, 0);
    $model->set ($iter, 0, !$val); # Always set the opposite of val to act as a toggle
}

sub editsys_mem() {
    my ($widget, $focus, $adjust) = @_;
    IMachine_setMemorySize($EIMachineM, int($adjust->get_value()));
    return 0; # Must return this value for the signal used.
}

sub editsys_apic() {
    my ($widget, $focus) = @_;
    my $IBIOSSettings = IMachine_getBIOSSettings($EIMachineM);
    IBIOSSettings_setIOAPICEnabled($IBIOSSettings, $widget->get_active());
}

sub editsys_efi() {
    my ($widget, $focus) = @_;
    if ($widget->get_active() == 1) { IMachine_setFirmwareType($EIMachineM, 'EFI'); }
    else { IMachine_setFirmwareType($EIMachineM, 'BIOS'); }
}

sub editsys_processors() {
    my ($widget, $focus, $adjust) = @_;
    IMachine_setCPUCount($EIMachineM, int($adjust->get_value()));
    return 0; # Must return this value for the signal used.
}

sub editsys_pae() {
    my ($widget, $focus) = @_;
    IMachine_setCPUProperty($EIMachineM, 'PAE', $widget->get_active());
}

sub editsys_vtx() {
    my ($widget, $focus) = @_;
    IMachine_setHWVirtExProperty($EIMachineM, 'Enabled', $widget->get_active());
}

sub editsys_utc() {
    my ($widget, $focus) = @_;
    IMachine_setRTCUseUTC($EIMachineM, $widget->get_active());
}

sub editsys_pointer() {
    my ($widget) = @_;
    IMachine_setPointingHidType($EIMachineM, &get_selected($widget, 1));
}

sub editsys_nested() {
    my ($widget, $focus) = @_;
    IMachine_setHWVirtExProperty($EIMachineM, 'NestedPaging', $widget->get_active());
}

sub editsys_boot_up() {
    my ($widget, $treeview) = @_;
    my ($model, $iter) = $treeview->get_selection->get_selected;
    my $path = $model->get_path($iter);
    $path->prev;
    my $previter = $model->get_iter($path);
    $model->move_before($iter, $previter);
}

sub editsys_boot_down() {
    my ($widget, $treeview) = @_;
    my ($model, $iter) = $treeview->get_selection->get_selected;
    my $nextiter = $model->iter_next($iter);
    $model->move_after($iter, $nextiter) if ($nextiter);
}

sub editsys_bootorder() {
    my ($widget, $treeview) = @_;
    my $model = $treeview->get_model();
    my $iter = $model->get_iter_first();
    my $i = 1;

    while ($iter) {
        my $dev = $model->get($iter,1);
        $dev = 'Null' if ($model->get($iter,0) == 0);
        IMachine_setBootOrder($EIMachineM, $i, $dev);
        $iter = $model->iter_next($iter);
        $i++;
    }
}

sub editsys_boot_selected() {
    &sensitive_on($gui{buttonEditSysBootDown}, $gui{buttonEditSysBootUp});
}

sub editdisp_vidmem() {
    my ($widget, $focus, $adjust) = @_;
    IMachine_setVRAMSize($EIMachineM, int($adjust->get_value()));
    return 0; # Must return this value for the signal used.
}

sub editdisp_monitors() {
    my ($widget, $focus, $adjust) = @_;
    IMachine_setMonitorCount($EIMachineM, int($adjust->get_value()));
    return 0; # Must return this value for the signal used.
}

sub editdisp_2D() {
    my ($widget, $focus) = @_;
    IMachine_setAccelerate2DVideoEnabled($EIMachineM, $widget->get_active());
}

sub editdisp_3D() {
    my ($widget, $focus) = @_;
    IMachine_setAccelerate3DEnabled($EIMachineM, $widget->get_active());
}

sub editdisp_RDP() {
    my ($widget, $focus) = @_;
    my $IVRDPServer = IMachine_getVRDPServer($EIMachineM);
    IVRDPServer_setEnabled($IVRDPServer, $widget->get_active());
}

sub editdisp_RDPTime() {
    my ($widget) = @_;
    my $authtime = int($widget->get_text());
    my $IVRDPServer = IMachine_getVRDPServer($EIMachineM);
    IVRDPServer_setAuthTimeout($IVRDPServer, $authtime) if ($authtime);
}

sub editdisp_RDPPort() {
    my ($widget) = @_;
    my $ports = $widget->get_text();
    # Ideally some additional validation would be needed here
    my $IVRDPServer = IMachine_getVRDPServer($EIMachineM);
    IVRDPServer_setPorts($IVRDPServer, $ports) if ($ports);
}

sub editdisp_RDPAuth() {
    my ($widget) = @_;
    my $IVRDPServer = IMachine_getVRDPServer($EIMachineM);
    IVRDPServer_setAuthType($IVRDPServer, &get_selected($widget, 1));
}

sub editdisp_RDPMulti() {
    my ($widget, $focus) = @_;
    my $IVRDPServer = IMachine_getVRDPServer($EIMachineM);
    IVRDPServer_setAllowMultiConnection($IVRDPServer, $widget->get_active());
}

sub editstor_tree_fill() {
    my ($IMachine) = @_;
    &busy_window($gui{dialogEdit});
    &sensitive_off(@{ $sensgrp{stor_unsel} });
    &clr_lstore($gui{treestoreEditStor});
    my @IStorageController = IMachine_getStorageControllers($IMachine);

    foreach my $controller (@IStorageController) {
        my $cname = IStorageController_getName($controller);
        my $bus = IStorageController_getBus($controller);
        my $iter = $gui{treestoreEditStor}->append(undef);
        $gui{treestoreEditStor}->set($iter, (0 => $cname,
                                             1 => $cname,
                                             2 => $bus . ' Controller',
                                             3 => $controller,
                                             7 => $cname));

        my @IMediumAttachment = IMachine_getMediumAttachmentsOfController($IMachine, $cname);

        foreach my $attachment (@IMediumAttachment) {
            my $citer = $gui{treestoreEditStor}->append($iter);
            ## Needs fixing - does it???
            if ($$attachment{medium}) {
                my $baseIMedium = IMedium_getBase($$attachment{medium});
                my $mname = IMedium_getName($baseIMedium); # Use the
                my $prettyname = $mname;
                if ($baseIMedium ne $$attachment{medium}) { $prettyname = "(*) $prettyname"; }
                $gui{treestoreEditStor}->set($citer, (0 => $mname,
                                                      1 => $cname,
                                                      2 => $$attachment{type},
                                                      3 => $$attachment{medium},
                                                      4 => $$attachment{device},
                                                      5 => $$attachment{port},
                                                      6 => IMedium_getLocation($$attachment{medium}),
                                                      7 => $prettyname));
            }
            else {
                $gui{treestoreEditStor}->set($citer, (0 => '<empty>',
                                                      1 => $cname,
                                                      2 => $$attachment{type},
                                                      3 => '<empty>',
                                                      4 => $$attachment{device},
                                                      5 => $$attachment{port},
                                                      6 => '<empty>',
                                                      7 => '<empty>'));
            }
        }
    }
    $gui{treeviewEditStor}->expand_all();
    &unbusy_window($gui{dialogEdit});
}


sub editstor_ctr_add() {
    my ($widget) = @_;
    my $bus = 'Floppy'; # Assume floppy unless set otherwise

    if ($widget eq $gui{menuitemCtrAddIDE}) { $bus = 'IDE'; }
    elsif ($widget eq $gui{menuitemCtrAddSCSI}) { $bus = 'SCSI'; }
    elsif ($widget eq $gui{menuitemCtrAddSATA}) { $bus = 'SATA'; }
    elsif ($widget eq $gui{menuitemCtrAddSAS}) { $bus = 'SAS'; }

    my @IStorageController = IMachine_getStorageControllers($EIMachineM);
    my $exists = 0;

    foreach my $ctr (@IStorageController) { $exists = 1 if (IStorageController_getBus($ctr) eq $bus) }

    if (!$exists) {
        IMachine_addStorageController($EIMachineM, "$bus Controller", $bus);
        &editstor_tree_fill($EIMachineM);
    }
    elsif ($exists) { &show_err_msg('ctrallocated', $gui{messagedialogError}); }
}



sub editstor_attach_add() {
    my ($widget, $IMedium) = @_;
    my $mtype = IMedium_getDeviceType($IMedium);
    my ($cname, $cname2, $ctype, $IStorCtr) = &get_selected($gui{treeviewEditStor});
    my ($freedev, $freeport) = &get_free_deviceport($EIMachineM, $IStorCtr);
    my @IMediumAttachment = IMachine_getMediumAttachmentsOfController($EIMachineM, $cname);
    my $muuid = IMedium_getId($IMedium);
    my $attached = 0;

    # Check for an empty Floppy/DVD first and try an use that (if medium appropriate)
    if ($mtype eq 'DVD') {
        foreach my $attach (@IMediumAttachment) {
            next if ($$attach{medium}); # We ignore if it already has a medium
            next if ($$attach{type} ne 'DVD'); # Only interested in DVD devices
            IMachine_mountMedium($EIMachineM, $cname, $$attach{port}, $$attach{device}, $muuid);
            $attached = 1;
        }
    }
    elsif ($mtype eq 'Floppy') {
        foreach my $attach (@IMediumAttachment) {
            next if ($$attach{medium}); # We ignore if it already has a medium
            next if ($$attach{type} ne 'Floppy'); # Only interested in DVD devices
            IMachine_mountMedium($EIMachineM, $cname, $$attach{port}, $$attach{device}, $muuid);
            $attached = 1;
        }

    }

    # For non removable media or if no empty Floppy/DVD, then add new attachment
    if ($attached == 0) {
        if ($freedev ne -1) {
            &create_sas_sata_port($IStorCtr); # Harmless if not SAS/SATA controller
            IMachine_attachDevice($EIMachineM, $cname, $freeport, $freedev, $mtype, $muuid);
            $attached = 1
        }
    }

    # If medium wasn't attached, controller must be full
    if (!$attached) { &show_err_msg('ctrfull', $gui{messagedialogError}); }
    else { &editstor_tree_fill($EIMachineM); }
}

sub editstor_attach_rem() {
    my ($widget, $treeview) = @_;
    my ($mname, $cname, $mtype, $IMedium, $mdev, $mport) = &get_selected($treeview);

    # Unmount removable media, otherwise remove device for everything.
    if ((($mtype eq 'Floppy') or ($mtype eq 'DVD')) and ($IMedium ne '<empty>')) {
        IMachine_mountMedium($EIMachineM, $cname, $mport, $mdev, 'UUID 00000000-0000-0000-0000-000000000000');
    }
    else { IMachine_detachDevice($EIMachineM, $cname, $mport, $mdev); }

    &editstor_tree_fill($EIMachineM);
}

sub editstor_ctr_rem() {
    my ($widget, $treeview) = @_;
    my $ctrname = &get_selected($treeview, 0);
    my @IMediumAttachments = IMachine_getMediumAttachmentsOfController($EIMachineM, $ctrname);

    if (@IMediumAttachments) { &show_err_msg('ctrinuse', $gui{messagedialogWarning}); }
    else { IMachine_removeStorageController($EIMachineM, $ctrname); }
    &editstor_tree_fill($EIMachineM);
}

sub editstor_selected() {
    my ($widget, $entryctr) = @_;
    my ($cname, $cname2, $ctype, $IStorCtr) = &get_selected($widget);
    if ($ctype =~ m/Controller/) {
        &sensitive_on(@{ $sensgrp{ctr_sel_on} });
        &sensitive_off($gui{buttonEditStorRemoveAttach});
        $entryctr->set_text($cname);
        &editstor_ctrtype_fill($ctype, $IStorCtr);

        if ($ctype eq 'IDE Controller') {
            &sensitive_on($gui{menuitemAttachHD}, $gui{menuitemAttachDVD});
            &sensitive_off($gui{menuitemAttachFloppy});
        }
        elsif ($ctype eq 'Floppy Controller') {
            &sensitive_off($gui{menuitemAttachHD}, $gui{menuitemAttachDVD});
            &sensitive_on($gui{menuitemAttachFloppy});
        }
        else {
            &sensitive_on($gui{menuitemAttachHD});
            &sensitive_off($gui{menuitemAttachFloppy}, $gui{menuitemAttachDVD});
        }
    }
    else {
        &sensitive_on($gui{buttonEditStorRemoveAttach});
        &sensitive_off(@{ $sensgrp{ctr_sel_off} });
    }
}

sub editstor_ctrtype_fill() {
    my ($ctype, $IStorCtr) = @_;
    $gui{comboboxEditStorCtrType}->signal_handler_block($signal{stortype});
    &clr_lstore($gui{liststoreEditStorCtrType});
    my $cvariant = IStorageController_getControllerType($IStorCtr);

    if ($ctype eq 'IDE Controller') {
        &append_lstore($gui{liststoreEditStorCtrType}, (0 => 'PIIX3'));
        &append_lstore($gui{liststoreEditStorCtrType}, (0 => 'PIIX4'));
        &append_lstore($gui{liststoreEditStorCtrType}, (0 => 'ICH6'));
    }
    elsif ($ctype eq 'SATA Controller') {
        &append_lstore($gui{liststoreEditStorCtrType}, (0 => 'IntelAhci'));
    }
    elsif ($ctype eq 'SAS Controller') {
        &append_lstore($gui{liststoreEditStorCtrType}, (0 => 'LsiLogicSas'));
    }
    elsif ($ctype eq 'SCSI Controller') {
        &append_lstore($gui{liststoreEditStorCtrType}, (0 => 'LsiLogic'));
        &append_lstore($gui{liststoreEditStorCtrType}, (0 => 'BusLogic'));
    }
    elsif ($ctype eq 'Floppy Controller') {
        &append_lstore($gui{liststoreEditStorCtrType}, (0 => 'I82078'));
    }

    if ($cvariant eq 'ICH6') { $gui{comboboxEditStorCtrType}->set_active(2); }
    elsif ($cvariant eq 'PIIX4' || $cvariant eq 'BusLogic') { $gui{comboboxEditStorCtrType}->set_active(1); }
    else { $gui{comboboxEditStorCtrType}->set_active(0); }

    $gui{checkbuttonEditStorCache}->set_active($ETruth{IStorageController_getUseHostIOCache($IStorCtr)});
    $gui{comboboxEditStorCtrType}->signal_handler_unblock($signal{stortype});
}

sub editstor_ctrtype() {
    my ($widget, $treeview) = @_;
    my ($cname, $cname2, $ctype, $IStorCtr) = &get_selected($treeview);
    IStorageController_setControllerType($IStorCtr, &get_selected($widget, 0));
    &editstor_tree_fill($EIMachineM);
}

# Set/Clear the host I/O cache for the controller
sub editstor_ctrcache() {
    my ($widget, $treeview) = @_;
    my ($cname, $cname2, $ctype, $IStorCtr) = &get_selected($treeview);
    IStorageController_setUseHostIOCache($IStorCtr, $widget->get_active());
}

# Enable/Disable the audio controller
sub editaudio() {
    my ($widget, $focus) = @_;
    my $IAudioAdapter = IMachine_getAudioAdapter($EIMachineM);
    IAudioAdapter_setEnabled($IAudioAdapter, $widget->get_active());
}

sub editaudio_ctr() {
    my ($widget) = @_;
    my $IAudioAdapter = IMachine_getAudioAdapter($EIMachineM);
    my $controller = &get_selected($widget, 1);
    IAudioAdapter_setAudioController($IAudioAdapter, $controller);
}

sub editaudio_driver() {
    my ($widget) = @_;
    my $IAudioAdapter = IMachine_getAudioAdapter($EIMachineM);
    my $driver = &get_selected($widget, 1);
    IAudioAdapter_setAudioDriver($IAudioAdapter, $driver);
}

sub editnet_adapter() {
    my ($widget, $focus, $checkbutton) = @_;
    my ($INetworkAdapter, $num) = &callout_getnetadapter($checkbutton);
    INetworkAdapter_setEnabled($INetworkAdapter, $widget->get_active());
}

sub editnet_cable() {
    my ($widget, $focus, $checkbutton) = @_;
    my ($INetworkAdapter, $num) = &callout_getnetadapter($checkbutton);
    INetworkAdapter_setCableConnected($INetworkAdapter, $widget->get_active());
}

sub editnet_mac() {
    my ($widget, $focus, $checkbutton) = @_;
    my ($INetworkAdapter, $num) = &callout_getnetadapter($checkbutton);
    my $mac = $widget->get_text();
    INetworkAdapter_setMACAddress($INetworkAdapter, $mac) if (length($mac) == 12);
}

sub editnet_adapter_type() {
    my ($widget, $checkbutton) = @_;
    my ($INetworkAdapter, $num) = &callout_getnetadapter($checkbutton);
    my $type = &get_selected($widget, 1);
    INetworkAdapter_setAdapterType($INetworkAdapter, $type);
}

sub editnet_attach() {
    my ($widget, $checkbutton) = @_;
    my ($INetworkAdapter, $num) = &callout_getnetadapter($checkbutton);
    $gui{'comboboxEditNetName' . $num}->signal_handler_block($signal{'netname' . $num});
    my $IHost = IVirtualBox_getHost($gui{websn});
    my $AttachType = &get_selected($widget, 1);
    &clr_lstore($gui{'liststoreEditNetName' . $num});

    if ($AttachType eq 'Bridged') {
        # Set current bridged network name for NICs 1-4, depending on attachment type
        my @IHostNetworkInterface = IHost_getNetworkInterfaces($IHost);
        foreach my $interface (@IHostNetworkInterface) {
            if (IHostNetworkInterface_getInterfaceType($interface) eq 'Bridged') {
                my $name = IHostNetworkInterface_getName($interface);
                my $iter = &append_lstore($gui{'liststoreEditNetName' . $num}, (0 => $name));
            }
        }
        INetworkAdapter_attachToBridgedInterface($INetworkAdapter);
        &hide_off($gui{'labelEditNetName' . $num}, $gui{'comboboxEditNetName' . $num});
        &hide_on($gui{'labelEditNetNameInt' . $num}, $gui{'entryEditNetNameInt' . $num});
    }
    elsif ($AttachType eq 'HostOnly') {
        # Set current hostonly network name for NICs 1-4, depending on attachment type
        my @IHostNetworkInterface = IHost_getNetworkInterfaces($IHost);
        foreach my $interface (@IHostNetworkInterface) {
            if (IHostNetworkInterface_getInterfaceType($interface) eq 'HostOnly') {
                my $name = IHostNetworkInterface_getName($interface);
                my $iter = &append_lstore($gui{'liststoreEditNetName' . $num}, (0 => $name));
            }
        }
        INetworkAdapter_attachToHostOnlyInterface($INetworkAdapter);
        &hide_off($gui{'labelEditNetName' . $num}, $gui{'comboboxEditNetName' . $num});
        &hide_on($gui{'labelEditNetNameInt' . $num}, $gui{'entryEditNetNameInt' . $num});
    }
    elsif ($AttachType eq 'Internal') {
        my $name = INetworkAdapter_getInternalNetwork($INetworkAdapter);
        $gui{'entryEditNetNameInt' . $num}->set_text($name);
        INetworkAdapter_attachToInternalNetwork($INetworkAdapter);
        &hide_on($gui{'labelEditNetName' . $num}, $gui{'comboboxEditNetName' . $num});
        &hide_off($gui{'labelEditNetNameInt' . $num}, $gui{'entryEditNetNameInt' . $num});
    }
    elsif ($AttachType eq 'NAT') {
        INetworkAdapter_attachToNAT($INetworkAdapter);
        &hide_on($gui{'labelEditNetName' . $num}, $gui{'comboboxEditNetName' . $num},
                 $gui{'labelEditNetNameInt' . $num}, $gui{'entryEditNetNameInt' . $num});
    }
    else {
        # Assume detach as we either don't support it yet, or it really is detach
        INetworkAdapter_detach($INetworkAdapter);
        &hide_on($gui{'labelEditNetName' . $num}, $gui{'comboboxEditNetName' . $num},
                 $gui{'labelEditNetNameInt' . $num}, $gui{'entryEditNetNameInt' . $num});
    }

    $gui{'comboboxEditNetName' . $num}->signal_handler_unblock($signal{'netname' . $num});
    $gui{'comboboxEditNetName' . $num}->set_active(0); # This will call editnet_name()
}

sub editnet_name() {
    my ($widget, $checkbutton) = @_;
    my ($INetworkAdapter, $num) = &callout_getnetadapter($checkbutton);
    my $AttachType = &get_selected($gui{'comboboxEditNetAttach' . $num}, 1);
    if ($AttachType eq 'Bridged' | $AttachType eq 'HostOnly') {
        INetworkAdapter_setHostInterface($INetworkAdapter, $gui{'comboboxEditNetName' . $num}->get_active_text());
    }
}

sub editnet_name_internal() {
    my ($widget, $focus, $checkbutton) = @_;
    my ($INetworkAdapter, $num) = &callout_getnetadapter($checkbutton);
    my $name = $gui{'entryEditNetNameInt' . $num}->get_text();
    INetworkAdapter_setInternalNetwork($INetworkAdapter, $name) if ($name);
}

sub callout_getnetadapter() {
    my ($checkbutton) = @_;
    my $slot = 0;
    if ($checkbutton eq $gui{checkbuttonEditNetEnable1}) { $slot = 0; }
    elsif ($checkbutton eq $gui{checkbuttonEditNetEnable2}) { $slot = 1; }
    elsif ($checkbutton eq $gui{checkbuttonEditNetEnable3}) { $slot = 2; }
    elsif ($checkbutton eq $gui{checkbuttonEditNetEnable4}) { $slot = 3; }
    my $INetworkAdapter = IMachine_getNetworkAdapter($EIMachineM, $slot);
    return $INetworkAdapter, $slot+1;
}

sub callout_getserial() {
    my ($checkbutton) = @_;
    my $slot = 0;
    if ($checkbutton eq $gui{checkbuttonEditSerialEnable1}) { $slot = 0; }
    elsif ($checkbutton eq $gui{checkbuttonEditSerialEnable2}) { $slot = 1; }
    my $ISerialPort = IMachine_getSerialPort($EIMachineM, $slot);
    return $ISerialPort, $slot+1;
}

sub editserial() {
    my ($widget, $focus, $checkbutton) = @_;
    my ($ISerialPort, $num) = &callout_getserial($checkbutton);
    ISerialPort_setEnabled($ISerialPort, $widget->get_active());
}

sub editserial_irq() {
    my ($widget, $focus, $checkbutton) = @_;
    my $irq = int($widget->get_text());
    $irq = 255 if ($irq > 255);
    $irq = 15 if ($irq > 15 and !$gui{checkbuttonEditSysAPIC}->get_active());
    my ($ISerialPort, $num) = &callout_getserial($checkbutton);
    ISerialPort_setIRQ($ISerialPort, $irq);
}

sub editserial_ioport() {
    my ($widget, $focus, $checkbutton) = @_;
    my $ioport = $widget->get_text();
    $ioport = 'FFFF' if (hex($ioport) > 65535);

    if ($ioport) {
        my ($ISerialPort, $num) = &callout_getserial($checkbutton);
        ISerialPort_setIOBase($ISerialPort, hex($ioport));
    }
}

sub editserial_portmode() {
    my ($widget, $checkbutton) = @_;
    my ($ISerialPort, $num) = &callout_getserial($checkbutton);
    ISerialPort_setHostMode($ISerialPort, &get_selected($widget, 0));
}

sub editserial_createpipe() {
    my ($widget, $focus, $checkbutton) = @_;
    my ($ISerialPort, $num) = &callout_getserial($checkbutton);
    ISerialPort_setServer($ISerialPort, $widget->get_active());
}

sub editserial_portpath() {
    my ($widget, $focus, $checkbutton) = @_;
    my $path = $widget->get_text();
    if ($path) {
        my ($ISerialPort, $num) = &callout_getserial($checkbutton);
        ISerialPort_setPath($ISerialPort, $path);
    }
}

sub editserial_portnum() {
    my ($widget, $checkbutton) = @_;
    my $portnum = &get_selected($widget, 0);
    my ($ISerialPort, $num) = &callout_getserial($checkbutton);

    if ($portnum eq 'COM1') {
        $gui{'entryEditSerialIRQ' . $num}->set_text('4');
        $gui{'entryEditSerialIO' . $num}->set_text('3F8');
    }
    elsif ($portnum eq 'COM2') {
        $gui{'entryEditSerialIRQ' . $num}->set_text('3');
        $gui{'entryEditSerialIO' . $num}->set_text('2F8');
    }
    elsif ($portnum eq 'COM3') {
        $gui{'entryEditSerialIRQ' . $num}->set_text('4');
        $gui{'entryEditSerialIO' . $num}->set_text('3E8');
    }
    elsif ($portnum eq 'COM4') {
        $gui{'entryEditSerialIRQ' . $num}->set_text('3');
        $gui{'entryEditSerialIO' . $num}->set_text('2E8');
    }

    if ($portnum ne 'Custom') {
        &editserial_irq($gui{'entryEditSerialIRQ' . $num}, 0, $checkbutton);
        &editserial_ioport($gui{'entryEditSerialIO' . $num}, 0, $checkbutton);
        &sensitive_off($gui{'entryEditSerialIRQ' . $num}, $gui{'entryEditSerialIO' . $num});
    }
    else { &sensitive_on($gui{'entryEditSerialIRQ' . $num}, $gui{'entryEditSerialIO' . $num}); }
}

sub editusb() {
    my ($widget, $focus) = @_;
    my $IUSBController = IMachine_getUSBController($EIMachineM);
    IUSBController_setEnabled($IUSBController, $widget->get_active());
}

sub editusb_ehci() {
    my ($widget, $focus) = @_;
    my $IUSBController = IMachine_getUSBController($EIMachineM);
    IUSBController_setEnabledEhci($IUSBController, $widget->get_active());
}

sub tv_shared_fill() {
    my ($IMachine) = @_;
    &clr_lstore($gui{liststoreEditShared});
    &sensitive_off($gui{buttonEditSharedRemove}, $gui{buttonEditSharedEdit});
    my @ISharedFolder = IMachine_getSharedFolders($IMachine);
    foreach my $share (@ISharedFolder) {
        my $access = 'Full';
        $access = 'Read-Only' if ($$share{writable} eq 'false');
        &append_lstore($gui{liststoreEditShared}, (0 => $$share{name},
                                                   1 => $$share{hostPath},
                                                   2 => $access));
    }
}

sub remove_shared() {
    my ($widget, $treeview) = @_;
    my $name = &get_selected($treeview, 0);
    IMachine_removeSharedFolder($EIMachineM, $name);
    &tv_shared_fill($EIMachineM);
}

sub tv_shared_selected() {
    my ($widget) = @_;
    &sensitive_on($gui{buttonEditSharedRemove}, $gui{buttonEditSharedEdit});
}

1;