# Virtual Media Management
use strict;
use warnings;
our (%gui, %hostspec);

sub show_vmm_dialog() {
    my ($widget) = @_;
    &busy_window($gui{windowMain});
    &set_status_msg("Retrieving media information...");
    &vmm_mediahd_fill();
    &vmm_mediadvd_fill();
    &vmm_mediafloppy_fill();
    &set_status_msg("Retrieved media information.");
    $gui{dialogVMM}->run;
    $gui{dialogVMM}->hide;
    &unbusy_window($gui{windowMain});
}

sub show_createhd_dialog() {
    my ($widget) = @_;
    &busy_window($gui{dialogVMM});
    $gui{entryCreateHDName}->set_text('NewHardDisk' . int(rand(9999)));
    $gui{spinbuttonCreateHDSize}->set_range(8.00, $hostspec{maxhdsize});
    $gui{spinbuttonCreateHDSize}->set_value(2048.00);
    my $response = $gui{dialogCreateHD}->run;
    $gui{dialogCreateHD}->hide;
    my $diskname = $gui{entryCreateHDName}->get_text();
    $diskname =~ s/\.vdi$//;
    my $disksize = $gui{spinbuttonCreateHDSize}->get_value_as_int();

    if ($diskname and ($response eq 'ok')) {
        if ($gui{checkbuttonCreateHDDynamic}->get_active()) { &create_new_HD($diskname, 'VDI', 'Standard', $disksize); }
        else { &create_new_HD($diskname, 'VDI', 'Fixed', $disksize); }
        &vmm_mediahd_fill();
    }
    &unbusy_window($gui{dialogVMM});
}

sub recurse_media_snapshot() {
    my ($treestore, $IMedium, $iter, $type) = @_;
    my $citer = $treestore->append($iter);
    my $gnames;
    my $name = IMedium_getName($IMedium);
    my @guuids = IMedium_getMachineIds($IMedium);
    IMedium_refreshState($IMedium); # Needed otherwise sizes may be 0

    foreach my $id (@guuids) {
        my $snames;
        my $IMachine = IVirtualBox_getMachine($gui{websn}, $id);
        my $mname = IMachine_getName($IMachine);
        my @suuids = IMedium_getSnapshotIds($IMedium, $id);

        foreach my $snapid (@suuids) {
            next if ($snapid eq $id);
            my $ISnapshot = IMachine_getSnapshot($IMachine, $snapid);
            my $snapname = ISnapshot_getName($ISnapshot);
            $snames .= "$snapname, ";
        }

        if ($snames) {
            $snames =~ s/, $//;
            $gnames .= "$mname ($snames), ";}
        else { $gnames .= "$mname, "; }
    }

    $gnames = '<None>' if (!$gnames);
    $gnames =~ s/, $//; # Remove any trailing,

    if ($type eq 'HardDisk') {
        my $asize = int(IMedium_getSize($IMedium) / 1048576);
        my $vsize = IMedium_getLogicalSize($IMedium);
        $treestore->set($citer, (0 => $name, 1 => $IMedium, 2 => $gnames, 3 => "$asize (MiB)", 4=> "$vsize (MiB)"));
    }
    elsif ($type eq 'DVD') {
        my $size = int(IMedium_getSize($IMedium) / 1048576);
        $treestore->set($citer, (0 => $name, 1 => $IMedium, 2 => $gnames, 3 => "$size (MiB)"));
    }
    else {
        my $size = IMedium_getSize($IMedium);
        $treestore->set($citer, (0 => $name, 1 => $IMedium, 2 => $gnames, 3 => "$size (Bytes)"));
    }
    &recurse_media_snapshot($treestore, $_, $citer, $type) foreach (IMedium_getChildren($IMedium));
}


sub vmm_mediahd_fill() {
    &clr_lstore($gui{treestoreVMMHD});
    $gui{entryVMMAttachedTo}->set_text('');
    my %IMedium = &get_all_media('HardDisk');

    foreach my $hd (sort { lc($a) cmp lc($b) } (keys %IMedium)) {
        &recurse_media_snapshot($gui{treestoreVMMHD}, $IMedium{$hd}, undef, 'HardDisk');
    }
}

sub vmm_mediadvd_fill() {
    &clr_lstore($gui{treestoreVMMDVD});
    $gui{entryVMMAttachedTo}->set_text('');
    my %IMedium = &get_all_media('DVD');

    foreach my $dvd (sort { lc($a) cmp lc($b) } (keys %IMedium)) {
        &recurse_media_snapshot($gui{treestoreVMMDVD}, $IMedium{$dvd}, undef, 'DVD');
    }
}

sub vmm_mediafloppy_fill() {
    &clr_lstore($gui{treestoreVMMFloppy});
    $gui{entryVMMAttachedTo}->set_text('');
    my %IMedium = &get_all_media('Floppy');

    foreach my $floppy (sort { lc($a) cmp lc($b) } (keys %IMedium)) {
        &recurse_media_snapshot($gui{treestoreVMMFloppy}, $IMedium{$floppy}, undef, 'Floppy');
    }
}

sub vmm_selected() {

    my ($widget, $lstore) = @_;
    my ($name, $IMedium, $gnames) = &get_selected($widget);

    # Don't allow remove/release if it has sub-snapshots
    if (IMedium_getChildren($IMedium)) {
        &sensitive_off($gui{toolbuttonVMMRemove});
        &sensitive_off($gui{toolbuttonVMMRelease});
    }
    else {
        if ($gnames eq '<None>') {
            &sensitive_on($gui{toolbuttonVMMRemove});
            &sensitive_off($gui{toolbuttonVMMRelease});
        }
        else {
            &sensitive_off($gui{toolbuttonVMMRemove});
            &sensitive_on($gui{toolbuttonVMMRelease});
        }
    }

    $gui{entryVMMAttachedTo}->set_text($gnames);
}

sub vmm_tabchanged() {
    my ($widget, $focus, $page) = @_;
    $gui{entryVMMAttachedTo}->set_text('');
    &sensitive_off($gui{toolbuttonVMMRemove}, $gui{toolbuttonVMMRelease});
    #We need to re-evaluate selection on tab change
    if ($page == 0) {
        $gui{buttonVMMAdd}->set_label('_Add Hard Disk');
        $gui{labelVMMLocation}->set_text_with_mnemonic('HD _Location');
        my $selected = $gui{treeviewVMMHD}->get_selection->get_selected();
        &vmm_selected($gui{treeviewVMMHD}, $gui{liststoreVMMHD}) if ($selected);
    }
    elsif ($page == 1) {
        $gui{buttonVMMAdd}->set_label('_Add CD/DVD');
        $gui{labelVMMLocation}->set_text_with_mnemonic('CD/DVD _Location');
        my $selected = $gui{treeviewVMMDVD}->get_selection->get_selected();
        &vmm_selected($gui{treeviewVMMDVD}, $gui{liststoreVMMDVD}) if ($selected);
    }
    else {
        $gui{buttonVMMAdd}->set_label('_Add Floppy');
        $gui{labelVMMLocation}->set_text_with_mnemonic('Floppy _Location');
        my $selected = $gui{treeviewVMMFloppy}->get_selection->get_selected();
        &vmm_selected($gui{treeviewVMMFloppy}, $gui{liststoreVMMFloppy}) if ($selected);
    }


}

sub vmm_release() {
    my ($widget, $notebook) = @_;
    my $page = $notebook->get_current_page();
    my ($name, $IMedium);
    my $warn = 0;
    &busy_window($gui{dialogVMM});

    if ($page == 0) { ($name, $IMedium) = &get_selected($gui{treeviewVMMHD}); }
    elsif ($page == 1) { ($name, $IMedium) = &get_selected($gui{treeviewVMMDVD}); }
    else { ($name, $IMedium) = &get_selected($gui{treeviewVMMFloppy}); }

    if (IMedium_getChildren($IMedium)) { &show_err_msg('releasemedium', $gui{messagedialogWarning}); }
    else {
        my @guuids = IMedium_getMachineIds($IMedium);

        foreach my $id (@guuids) {
            my $IMachine = IVirtualBox_getMachine($gui{websn}, $id);
            my @IMediumAttachment = IMachine_getMediumAttachments($IMachine);

            foreach my $attach (@IMediumAttachment) {

                if ($$attach{medium} eq $IMedium) {
                    my ($ISession, $IMachineM) = &get_mutable_session($IMachine);

                    if ($ISession) {
                        IMachine_detachDevice($IMachineM, $$attach{controller}, $$attach{port}, $$attach{device});
                        IMachine_saveSettings($IMachineM);
                        ISession_close($ISession);
                    }
                    else { $warn = 1; }
                }
            }
        }
    }

    if ($warn == 1) {
        &show_err_msg('mediuminuse', $gui{messagedialogWarning});
        &set_status_msg("Storage $name was not released from all guests.");
    }
    else { &set_status_msg("Storage $name released."); }

    if ($page == 0) { &vmm_mediahd_fill(); }
    elsif ($page == 1) { &vmm_mediadvd_fill(); }
    else { &vmm_mediafloppy_fill(); }

    &unbusy_window($gui{dialogVMM});
}

sub vmm_rem() {
    my ($widget, $notebook) = @_;
    my $page = $notebook->get_current_page();
    &busy_window($gui{dialogVMM});

    if ($page == 0) {
        my ($name, $IMedium) = &get_selected($gui{treeviewVMMHD});

        if (IMedium_getChildren($IMedium)) { &show_err_msg('removemedium', $gui{messagedialogWarning}); }
        else {
            &busy_window($gui{dialogVMM});
            my $response = $gui{dialogRemoveDelete}->run;
            $gui{dialogRemoveDelete}->hide;

            if ($response eq '1') { # Delete
                &set_status_msg("Deleting hard disk $name...");
                my $IProgress = IMedium_deleteStorage($IMedium);

                do {
                    &set_status_msg('Deleting hard disk... ' . IProgress_getPercent($IProgress) . '% complete.');
                    sleep 1; # Poll only every 1 second
                } until (IProgress_getCompleted($IProgress) eq 'true');

                &set_status_msg("Hard disk $name deleted.");
                &vmm_mediahd_fill();
            }
            elsif ($response eq '2') { # Remove
                IMedium_close($IMedium);
                &set_status_msg("Storage $name removed.");
                &vmm_mediahd_fill();
            }
            &unbusy_window($gui{dialogVMM});
        }
    }
    elsif ($page == 1) {
        my ($name, $IMedium) = &get_selected($gui{treeviewVMMDVD});
        IMedium_close($IMedium);
        &set_status_msg("Storage $name removed.");
        &vmm_mediadvd_fill();
    }
    else {
        my ($name, $IMedium) = &get_selected($gui{treeviewVMMFloppy});
        IMedium_close($IMedium);
        &set_status_msg("Storage $name removed.");
        &vmm_mediafloppy_fill();
    }
    &unbusy_window($gui{dialogVMM});
}

sub vmm_add() {
    my ($widget, $pathentry) = @_;
    &busy_window($gui{dialogVMM});
    my $location = $pathentry->get_text();
    $pathentry->set_text('');
    my $page = $gui{notebookVMM}->get_current_page();

    if ($location) {
        if ($page == 0) {
            IVirtualBox_openHardDisk($gui{websn}, $location, 'ReadWrite', 0, '', 0, '');
            &vmm_mediahd_fill();
        }
        elsif ($page == 1) {
            IVirtualBox_openDVDImage($gui{websn}, $location, undef);
            &vmm_mediadvd_fill();
        }
        else {
            IVirtualBox_openFloppyImage($gui{websn}, $location, undef);
            &vmm_mediafloppy_fill();
        }
    }

    &unbusy_window($gui{dialogVMM});
}

1;